*****************************************************************************
* Modulname:	Inferential Statistics
* Modultype:	C-ApLet
* Dest.Comp.:	HP38G
* Language:	System RPL
* Author:	2K/09/10, Detlef Mueller, Bellerbek 33, 22559 Hamburg, Germany
* Interface:
* Description:	Bases on Colin Croft's Inference Statistics program
*		set Infernce.prg & GetVals.prg
* To do:
*
* Copyright:	(c) 2000 by Detlef Mueller. ALL rights reserved.
*
* $Id: infer.s,v 1.8 2005/01/06 18:09:18 Detlef Exp $
*
* $Log: infer.s,v $
* Revision 1.8  2005/01/06 18:09:18  Detlef
* Changed a L14 reference to L11.
*
* Revision 1.7  2000/09/17 05:36:14  detlef
* Removed VIEWS Reset entry, changed the error handler again (w/o luck yet..),
* changed the display back to show big numbers, added some code to deal w/
* leading -, moved Range: for != to row 7 (for a cleaner display), tried to
* shorten the code further by introducing more INT_.., added %ABS for the
* +/- values, preset all Ln variables, fixed a typo in the preset Screen.
*
* Revision 1.6  2000/09/15 09:27:03  detlef
* Commented ShowStack, changed the method results are displayed: the even
* strings in Screen are now converted to the very small font, changed most
* rounding to -11, reviewed all result messages for typos etc.
*
* Revision 1.5  2000/09/15 04:03:22  detlef
* Reviewed InputL and its calls, made the alt. hypothesis choose contens
* depending on the distribution, made ENTER key starting a new inf. analy.
* as well, changed DoNew error catching, added Reset entry to VIEWS menu.
*
* Revision 1.4  2000/09/13 07:50:55  detlef
* Added stage display, fixed things shown up by running through Colin's
* test suite, fixed InputL cancel, removed ShowStack calls. ShowStack
* still inluded for later debugging. Reached alpha stage.
*
* Revision 1.3  2000/09/13 03:34:50  detlef
* Reworked FnRootX, added ChooseSimp, other minor changes. Alpha stage,
* still contains ShowStack calls.
*
* Revision 1.2  2000/09/12 23:45:47  detlef
* 1st aplha sketch -- FnRoot doesn't work as expected yet..
*
* Revision 1.1  2000/09/11 04:16:57  detlef
* Initial revision
*
*****************************************************************************

    TITLE	Inference


*****************************************************************************
*		INCLUDE FILES
*****************************************************************************

DEFINE	topic_CAPLET	topic_CAPLE
DEFINE	view23_CAPLET	view23_CAPL
DEFINE	view45_CAPLET	view45_CAPL
DEFINE	view7_CAPLET	view7_CAPLE

    INCLUDE	common38.h


*****************************************************************************
*		APLET LAYOUT
*****************************************************************************

ASSEMBLE
	Dir_Head 9,Inference,602
*>>>		 ^
*		 +-- Length of the following name (in chars)

~view01_CApLet	EQU	(#602)+4096*0
RPL

EXTERNAL	view01_CApLet		( tType for taking over Plot View   )
EXTERNAL	view6_CAPLET		( tType for taking over Note View   )
EXTERNAL	view7_CAPLET		( tType for taking over Sketch View )

EXTERNAL	NullMenuKey		( On the '38G, the NMK is a ROMPTR  )


	{				( tType variable, 1st var in dir    )
	    topic_CAPLET		( Topic type -- embedded code aplet )
	    view01_CApLet		( Plot type  -- points to our lib   )
	    view23_CAPLET		( Table type )
	    view45_CAPLET		( Symb type )
	    xDFLTNOTE			( Note type )
	    xDFLTPICT			( Pict type )
	}


    INCLUDE	share38.h		( Minimal variable set neccessary   )

ASSEMBLE
	Vfield  L34,IntTable

	CON(5)	=DOHSTR			Linktable variable, provides links
	REL(5)  endLink			to the objects w/in the next variable

	Link	_Main

	Link	_MainDisp
	Link	_MainKeys
	Link	_MainMenu
	Link	_New

	Link	_DoNew
	Link	_ChooseSimp
	Link	_SuccCnt$
	Link	_Sample$
	Link	_Pop$
	Link	_StdDev$
	Link	_DoRnd
	Link	_2NULL$
	Link	_Range

	Link	_InputL
	Link	_FnRootX

**	Link	_ShowStack
endLink
RPL

DEFINE	Main		INT_00

DEFINE	MainDisp	INT_01
DEFINE	MainKeys	INT_02
DEFINE	MainMenu	INT_03
DEFINE	New		INT_04

DEFINE	DoNew		INT_05
DEFINE	ChooseSimp	INT_06
DEFINE	SuccCnt$	INT_07
DEFINE	Sample$		INT_08
DEFINE	Pop$		INT_09
DEFINE	StdDev$		INT_0A
DEFINE	DoRnd		INT_0B
DEFINE	2NULL$		INT_0C
DEFINE	Range		INT_0D

DEFINE	InputL		INT_0E
DEFINE	FnRootX		INT_0F

** DEFINE	ShowStack	INT_10


ASSEMBLE
	Vfield  IntTable,DirVar

	CON(5)	=DOHSTR			Code variable, containes the sRPL
	REL(5)  codeEnd			objects encapsulated in a HSTR
RPL


*****************************************************************************
*		DEFINES
*****************************************************************************

ASSEMBLE
***		Some unsupported but usefull entries

=#4-		EQU	#DF143
=argtypeerr	EQU	#202	"Bad Argument Type"

=InputIFSpecs	EQU	#41BDA
=exprparse	EQU	#4E3EF
=STATUNRM	EQU	#760B3	( 3real UTPN )
=STATUSTT	EQU	#7619D	( 2real UTPT )
=paintparse	EQU	#D3BDD
=SETATTNERR	EQU	#A114A

=DirRNDXY	EQU	#237BE

RPL

DEFINE	RNDXY		DirRNDXY	( RNDXY's Jumptable entry is broken! )

DEFINE	FOURTY		FORTY		( Well... )

DEFINE	kpShift		TWO
DEFINE	kpAShft		FIVE
DEFINE	kcShift		THIRTYFIVE


DEFINE	@Par		LastBut3
DEFINE	!Par		LastBut3 REPLACE DROP

DEFINE	Screen@		TopicVar1@
DEFINE	Screen!		TopicVar1!
DEFINE	Method@		TopicVar2@
DEFINE	Method!		TopicVar2!
DEFINE	Type@		TopicVar3@
DEFINE	Type!		TopicVar3!
DEFINE	AltHyp@		TopicVar4@
DEFINE	AltHyp!		TopicVar4!
DEFINE	L1@		TopicVar5@
DEFINE	L1!		TopicVar5!
DEFINE	L2@		TopicVar6@
DEFINE	L2!		TopicVar6!
DEFINE	L3@		TopicVar7@
DEFINE	L3!		TopicVar7!
DEFINE	L4@		TopicVar8@
DEFINE	L4!		TopicVar8!
DEFINE	L5@		TopicVar9@
DEFINE	L5!		TopicVar9!
DEFINE	L6@		TopicVar10@
DEFINE	L6!		TopicVar10!
DEFINE	L7@		TopicVar11@
DEFINE	L7!		TopicVar11!
DEFINE	L8@		TopicVar12@
DEFINE	L8!		TopicVar12!
DEFINE	L9@		TopicVar13@
DEFINE	L9!		TopicVar13!
DEFINE	L10@		TopicVar14@
DEFINE	L10!		TopicVar14!
DEFINE	L11@		TopicVar15@
DEFINE	L11!		TopicVar15!
DEFINE	L12@		TopicVar16@
DEFINE	L12!		TopicVar16!
DEFINE	L13@		TopicVar17@
DEFINE	L13!		TopicVar17!
DEFINE	L14@		TopicVar18@
DEFINE	L14!		TopicVar18!
DEFINE	L15@		TopicVar19@
DEFINE	L15!		TopicVar19!
DEFINE	L16@		TopicVar20@
DEFINE	L16!		TopicVar20!
DEFINE	L17@		TopicVar21@
DEFINE	L17!		TopicVar21!
DEFINE	Q@		TopicVar22@
DEFINE	Q!		TopicVar22!
DEFINE	C@		TopicVar23@
DEFINE	C!		TopicVar23!

DEFINE	Z@		TopicVar24@
DEFINE	Z!		TopicVar24!
DEFINE	D@		TopicVar25@
DEFINE	D!		TopicVar25!
DEFINE	R@		TopicVar26@
DEFINE	R!		TopicVar26!
DEFINE	W@		TopicVar27@
DEFINE	W!		TopicVar27!
DEFINE	F@		TopicVar28@
DEFINE	F!		TopicVar28!
DEFINE	S@		TopicVar29@
DEFINE	S!		TopicVar29!
DEFINE	Y@		TopicVar30@
DEFINE	Y!		TopicVar30!
DEFINE	P@		TopicVar31@
DEFINE	P!		TopicVar31!
DEFINE	L@		TopicVar32@
DEFINE	L!		TopicVar32!
DEFINE	U@		TopicVar33@
DEFINE	U!		TopicVar33!
DEFINE	M@		TopicVar34@
DEFINE	M!		TopicVar34!
DEFINE	N@		TopicVar35@
DEFINE	N!		TopicVar35!
DEFINE	O@		TopicVar36@
DEFINE	O!		TopicVar36!

DEFINE	TV#L1		FIVE
DEFINE	TV#L2		SIX
DEFINE	TV#L3		SEVEN
DEFINE	TV#L4		EIGHT
DEFINE	TV#L5		NINE
DEFINE	TV#L6		TEN
DEFINE	TV#L7		ELEVEN
DEFINE	TV#L8		TWELVE
DEFINE	TV#L9		THIRTEEN
DEFINE	TV#L10		FOURTEEN
DEFINE	TV#L11		FIFTEEN
DEFINE	TV#L12		SIXTEEN
DEFINE	TV#L13		SEVENTEEN
DEFINE	TV#L14		EIGHTEEN
DEFINE	TV#L15		NINETEEN
DEFINE	TV#L16		TWENTY
DEFINE	TV#L17		TWENTYONE
DEFINE	TV#Q		TWENTYTWO
DEFINE	TV#C		TWENTYTHREE


*****************************************************************************
*		OBJECTS
*****************************************************************************


*** _Main *******************************************************************
* Interface :	( --> )
* Description :
*	Main program
* Contains :
*   NumUI	( --> ... )
*****************************************************************************

NAMELESS	_Main
::
    {   MINUSONE
	{				( This is the VIEWS menu	  )
	    {	"Enter Inference"
		:: TWO Main		( Enter via num view		  )
		;
	    }
	}
	StndVar
	StndEntry
	StndExit
    }
    BinLookup case EVAL
    DROP

    '					( View entry )
    ::					( *Load parameter into TopicVars* )
	@Par INNERDUP			( --> ... #n #n )

	ZERO_DO (DO)
	    SWAP OVER			( --> ... #n ob #n )
	    INDEX@#- TopicVarN!
	LOOP

	DROP
    ;
    '	 				( View exit )
    ::					( *Save TopicVars into parameter* )
	@Par LENCOMP DUP 		( --> #n )

	#1+_ONE_DO (DO)
	    INDEX@ TopicVarN@
	    SWAP			( --> ... ob #n )
	LOOP

	%0AllTopicVs			( Clear all topic vars )

	{}N !Par			( Store parameter into ApLet )

	SetDAsBad			( Signal bad display )
	DropJunk			( Drop any leftovers )
    ;
    ' MainDisp				( View display )
    ' MainKeys				( View hard key handler )
    TRUE				( Allow normal keys )
    MainMenu				( Initial menu )
    ONE 				( Initial menu page )
    '					( Error object )
    ::	DropJunk

	ERROR@
	MEMERR #=casedrop ERRJMP

	GETTHEMESG

	ERRORCLR ATTNFLGCLR FLUSHKEYS
	DoAlert
    ;
;

*** _MainDisp ***************************************************************
* Interface :	( --> )
* Description :
*	Main display handler
*****************************************************************************

NAMELESS	_MainDisp
::
    DA3OK?NOTIT ?DispMenu		( Redraw menu if not ok )

    DA1OK? KEYINBUFFER? OR ?SEMI	( Leave if DA1 is ok or key in buffer )

    RECLAIMDISP				( Clr rows 0-55 )

    Screen@ INNERCOMP TWO #/ SWAPDROP
    DUP ZERO_DO (DO)
					( --> $11 $12 .. $n1 $n2 #n )
	SWAP DUPTYPECSTR? ?SKIP DO>STR  ( --> .. $ # $' )
	ROTSWAP				( --> .. # $ $' )

	DUP CAR$ CHR \AD EQUAL
	IT
	::  SWAPDUP LEN$ 1_#1-SUB$ SWAP
	;

	&$ OVER INDEX@#- DISPN
    LOOP

    DROP

    SetDA1Valid				( Signal main display valid )
;

*** _MainKeys ***************************************************************
* Interface :	( --> )
* Description :
*	Main hardkey handler
*****************************************************************************

NAMELESS	_MainKeys
::
    kpNoShift #=casedrop		( Check/process unshifted keys )
    ::
	DUP#<7 casedrpfls		( Enable menukeys )

	kcShift #=casedrpfls		( Allow shift )

	kcEnter ?CaseKeyDef		( Enter -- new task )
	::  TakeOver
	    New
	;

	kcPlotView ?CaseKeyDef DoBadKey ( Disallow Plot/Symb/Num views )
	kcSymbView ?CaseKeyDef DoBadKey
	kcNumView  ?CaseKeyDef DoBadKey
	kcVarsMenu ?CaseKeyDef DoBadKey ( Disallow Vars/Math menus, )
	kcMathMenu ?CaseKeyDef DoBadKey ( they map in the menu bar  )

	DUP kpNoShift IsTaskSwKey? casedrpfls

	DROP 'DoBadKeyT			( Illegal key )
    ;

    kpShift #=casedrop			( Check/process shifted keys )
    ::
	kcShift #=casedrpfls		( Allow shift )

	kcOn #=casedrpfls		( Allow power off )

	kcPlotView ?CaseKeyDef DoBadKey ( Disallow Plot/Symb/Num views )
	kcSymbView ?CaseKeyDef DoBadKey
	kcNumView  ?CaseKeyDef DoBadKey
	kcLibrary  ?CaseKeyDef DoBadKey ( Disallow Views [maps in menu] )

	DUP kpLeftShift IsTaskSwKey? casedrpfls

	DROP 'DoBadKeyT			( Illegal key )
    ;

    2DUP IsTaskSwKey? case2drpfls	( Task-switch key not handled above ? )

    2DROP 'DoBadKeyT			( Illegal keyplane )
;

*** _MainMenu ***************************************************************
* Interface :	( --> )
* Description :
*	Main menu definition
*****************************************************************************

NAMELESS	_MainMenu
{					( **Initial menu** )
    NullMenuKey
    NullMenuKey				( Just some blank entries )
    NullMenuKey
    NullMenuKey
    NullMenuKey
    {
	"NEW"
	NAMELESS	_New
	::  TakeOver

	    DoNew

	    DropJunk
	    SetDA1Bad
	;
    }
}

*** _DoNew ******************************************************************
* Interface :	( --> )
* Description :
*	Do a new inferential analysis
*****************************************************************************

NAMELESS	_DoNew
::
    ( ** Query section ** )

    Method@
    "Inferential Method"
    {
	{ "Hypothesis Test" ONE }
	{ "Conf. Interval"  TWO }
    }

    NAMELESS	_ChooseSimp
    ::			( $Prompt { Choices } --> [ ChoicePart2 TRUE ] | FALSE )
	'DROPFALSE			( choose procedure )
	ROT				( top-of-list prompt )
**	FMT_P1&NONE			( object decompile format specification )
	SEVENTEEN
	4ROLL				( list of possible choices )
	5ROLL				( initial focus )
	Choose				( do choose )
	NOTcaseFALSE
	GetPart2 TRUE
    ;

    NOT?SEMI
    Method!

    Type@
    "Distrib. Statistic"
    {
	{ "Z-Test: 1 \B5" ONE }
	{ "Z-Test: \B51-\B52" TWO }
	{ "Z-Test: 1 P" THREE }
	{ "Z-Test: P1-P2" FOUR }
	{ "T-Test: 1 \B5" FIVE }
	{ "T-Test: \B51-\B52" SIX }
    }
    ChooseSimp NOT?SEMI
    Type!

    Method@ #1= IT
    ::
	AltHyp@
	"Alt. Hypothesis"

	Type@ #1- #2* #1+

	ARRY
	[
	    "\B5" "\B50"   "\B51" "\B52"  "\87" "\870"
	    "\871" "\872"  "\B5" "\B50"   "\B51" "\B52"
	]

	OVER #1+ OVER
	GETATELN DROP UNROT
	GETATELN DROPSWAP		( --> $l $r )

	2DUP CHR_< >H$ &$ UNROT
	2DUP CHR_> >H$ &$ UNROT
	CHR \8B    >H$ &$		( --> $< $> $!= )

	THREE #1+_ONE_DO (DO)
	    ROT "     " SWAP&$
	    INDEX@ TWO{}N
	LOOP

	THREE{}N

	ChooseSimp NOTcase :: RDROP ;
	AltHyp!
    ;

    {
	{ FALSE FALSE FALSE TRUE  FALSE FALSE TRUE  TRUE  FALSE FALSE FALSE FALSE FALSE TRUE  FALSE FALSE FALSE }
	{ FALSE FALSE FALSE FALSE TRUE  TRUE  FALSE FALSE TRUE  TRUE  FALSE FALSE FALSE FALSE TRUE  TRUE  FALSE }
	{ TRUE  FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE TRUE  FALSE FALSE TRUE  }
	{ FALSE TRUE  TRUE  FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE FALSE TRUE  TRUE  FALSE }
	{ FALSE FALSE FALSE TRUE  FALSE FALSE TRUE  FALSE FALSE FALSE TRUE  FALSE FALSE TRUE  FALSE FALSE FALSE }
	{ FALSE FALSE FALSE FALSE TRUE  TRUE  FALSE FALSE FALSE FALSE FALSE TRUE  TRUE  FALSE TRUE  TRUE  FALSE }
    }

    Type@ NTHCOMPDROP INNERCOMP reversym DROP

    Method@ #2= IT
    ::	Type@
	ONE   #=casedrop :: 7ROLL DROPFALSE 7UNROLL ;
	THREE #=casedrop :: SEVENTEEN ROLL DROPFALSE SEVENTEEN UNROLL ;
	FIVE  #=case     :: 7ROLL DROPFALSE 7UNROLL ;
    ;

    IT
    ::
	NAMELESS	_SuccCnt$
	    "Success count "

	"x:"
	TV#L1 InputL
    ;
    IT
    ::  SuccCnt$ "sample 1" &$
	"x1:"
	TV#L2 InputL
    ;
    IT
    ::  SuccCnt$ "sample 2" &$
	"x2:"
	TV#L3 InputL
    ;
    IT
    ::
	NAMELESS	_Sample$
	    "Sample "

	"mean" &$
	"\81:"
	TV#L4 InputL
    ;
    IT
    ::  Sample$ "1 mean" &$
	"\811:"
	TV#L5 InputL
    ;
    IT
    ::  Sample$ "2 mean" &$
	"\812:"
	TV#L6 InputL
    ;
    IT
    ::
	NAMELESS	_Pop$
	    "Population "

	" mean" &$
	"\B50:"
	TV#L7 InputL
    ;
    IT
    ::  Pop$

	NAMELESS	_StdDev$
	    "standard deviation"

	&$
	"\98:"
	TV#L8 InputL
    ;
    IT
    ::  Pop$ "1 " &$ StdDev$ &$
	"\981:"
	TV#L9 InputL
    ;
    IT
    ::  Pop$ "2 " &$ StdDev$ &$
	"\982:"
	TV#L10 InputL
    ;
    IT
    ::  Sample$ StdDev$ &$
	"Sx:"
	TV#L11 InputL
    ;
    IT
    ::  Sample$ "1 " &$ StdDev$ &$
	"S1:"
	TV#L12 InputL
    ;
    IT
    ::  Sample$ "2 " &$ StdDev$ &$
	"S2:"
	TV#L13 InputL
    ;
    IT
    ::  Sample$ "size" &$
	"n:"
	TV#L14 InputL
    ;
    IT
    ::  Sample$ "1 size" &$
	"n1:"
	TV#L15 InputL
    ;
    IT
    ::  Sample$ "2 size" &$
	"n2:"
	TV#L16 InputL
    ;
    IT
    ::  Pop$ "proportion" &$
	"\870:"
	TV#L17 InputL
    ;

    " level"
    Method@ #1=
    ITE
    ::  "Significance" SWAP&$
	"\8C:" TV#Q InputL
    ;
    ::  "Confidence" SWAP&$
	"C:"   TV#C InputL
    ;

    ( ** Calculating section ** )

    RECLAIMDISP

    "     Thinking..." THREE DISPN

    ::	Type@
	ONE #=casedrop
	::
	    L4@ L7@ 2DUP %> D!
	    %- L8@ L14@ %SQRT %/ %/ Z!

**	    (L0(4)-L0(7)) / (L0(8)/\v/L0(14)) \|>Z:
**	    L0(4)>L0(7)\|>D:
	;
	TWO #=casedrop
	::
	    L5@ L6@ %- DUP %0> D!
	    L9@ DUP %* L15@ %/ L10@ DUP %* L16@ %/ %+ %SQRT %/ Z!

**	    (L0(5)-L0(6)) / \v/(L0(9)\178 / L0(15)+L0(10)\178/L0(16))\|>Z:
**	    L0(5)-L0(6)>0\|>D:
	;
	THREE #=casedrop
	::
	    L1@ L14@ %/ DUP R!
	    L17@ 2DUP %> D! %-
	    L17@ %1 L17@ %- %* L14@ %/ %SQRT %/ Z!

**	    L0(1)/L0(14)\|>R:
**	    (R-L0(17))/\v/(L0(17)(1-L0(17))/L0(14))\|>Z:
**	    L0(1)/L0(14)>L0(17)\|>D:
	;
	FOUR #=casedrop
	::
	    L2@ L3@ %+ L15@ L16@ %+ %/ R!
	    L2@ L15@ %/ L3@ L16@ %/ %- DUP %0> D!
	    R@ %1 R@ %- %* %1 L15@ %/ %1 L16@ %/ %+ %* %SQRT %/ Z!

**	    (L0(2)+L0(3))/(L0(15)+L0(16))\|>R:
**	    (L0(2)/L0(15)-L0(3)/L0(16))/\v/(R*(1-R)(1/L0(15)+1/L0(16))\|>Z:
**	    L0(2)/L0(15)-L0(3)/L0(16)>0\|>D:
	;
	FIVE #=case
	::
	    L4@ L7@ 2DUP %> D! %-
	    L11@ L14@ %SQRT %/ %/ W!
	    L14@ %1 %- F!

**	    (L0(4)-L0(7))/(L0(11)/\v/L0(14))\|>W:
**	    L0(4)>L0(7)\|>D:
**	    L0(14)-1\|>F:
	;
					( SIX )
	L15@ %1 %- L12@ DUP %* %*
	L16@ %1 %- L13@ DUP %* %* %+
	L15@ L16@ %+ %2 %- DUP F! %/ %SQRT S!

	L5@ L6@ %- DUP %0> D!
	S@ %1 L15@ %/ %1 L16@ %/ %+ %SQRT %* %/ W!

**	\v/( ((L0(15)-1)L0(12)\178+(L0(16)-1)L0(13)\178)
**	 /(L0(15)+L0(16)-2) )\|>S:
**	(L0(5)-L0(6))/(S\v/(1/L0(15)+1/L0(16)))\|>W:
**	L0(5)-L0(6)>0\|>D:
**	L0(15)+L0(16)-2\|>F:
    ;

    Method@ #2=
    Type@ FOUR #<=
    ITE
    ::	case
	::
	    "UTPN(0,1,\ADX)-UTPN(0,1,X)-" C@ DO>STR &$ FnRootX Y!

**	    FNROOT(UTPN(0,1,-Y)-UTPN(0,1,Y)-C,Y,1)\|>Y:
	;

	AltHyp@

	ONE #=casedrop
	::
	    Z@ %1 %0 STATUNRM DO>STR "-X" &$ FnRootX P!
	    "UTPN(X,1,0)-" Q@ DO>STR &$ FnRootX Y!

**	    FNROOT(UTPN(Z,1,0)-P,P,1)\|>P:
**	    FNROOT(UTPN(Y,1,0)-Q,Y,1)\|>Y:
	;

	TWO #=case
	::
	    %0 %1 Z@ STATUNRM DO>STR "-X" &$ FnRootX P!
	    "UTPN(0,1,X)-" Q@ DO>STR &$ FnRootX Y!

**	    FNROOT(UTPN(0,1,Z)-P,P,1)\|>P:
**	    FNROOT(UTPN(0,1,Y)-Q,Y,1)\|>Y:
	;
					( THREE )
	D@ case
	::
	    %0 %1 Z@ STATUNRM DO>STR "-X" &$ FnRootX DUP %+ P!
	    "UTPN(0,1,X)-" Q@ DO>STR &$ "/2" &$ FnRootX Y!

**	    2FNROOT(UTPN(0,1,Z)-P,P,1)\|>P:
**	    FNROOT(UTPN(0,1,Y)-Q/2,Y,1)\|>Y:
	;

	Z@ %1 %0 STATUNRM DO>STR "-X" &$ FnRootX DUP %+ P!
	"UTPN(X,1,0)-" Q@ DO>STR &$ "/2" &$ FnRootX Y!

**	2FNROOT(UTPN(Z,1,0)-P,P,1)\|>P:
**	FNROOT(UTPN(Y,1,0)-Q/2,Y,1)\|>Y:
    ;
    ::	case
	::
	    F@ DO>STR
	    "UTPT(" OVER &$ ",\ADX)-UTPT(" &$ SWAP&$ ",X)-" &$
	    C@ DO>STR &$
	    FnRootX Y!

**	    FNROOT(UTPT(F,-Y)-UTPT(F,Y)-C,Y,1)\|>Y:
	;

	AltHyp@

	ONE #=casedrop
	::
	    %1 F@ W@ STATUSTT %- DO>STR "-X" &$ FnRootX P!
	    "1-UTPT(" F@ DO>STR &$ ",X)-" &$ Q@ DO>STR &$ FnRootX Y!

**	    FNROOT((1-UTPT(F,W))-P,P,1)\|>P:
**	    FNROOT((1-UTPT(F,Y))-Q,Y,1)\|>Y:
	;

	TWO #=case
	::
	    F@ W@ STATUSTT DO>STR "-X" &$ FnRootX P!
	    "UTPT(" F@ DO>STR &$ ",X)-" &$ Q@ DO>STR &$ FnRootX Y!

**	    FNROOT(UTPT(F,W)-P,P,1)\|>P:
**	    FNROOT(UTPT(F,Y)-Q,Y,1)\|>Y:
	;
					( THREE )
	D@ case
	::
	    F@ W@ STATUSTT DO>STR "-X" &$ FnRootX DUP %+ P!
	    "UTPT(" F@ DO>STR &$ ",X)-" &$ Q@ DO>STR &$ "/2" &$ FnRootX Y!

**	    2FNROOT(UTPT(F,W)-P,P,1)\|>P:
**	    FNROOT(UTPT(F,Y)-Q/2,Y,1)\|>Y:
	;

	%1 F@ W@ STATUSTT %- DO>STR "-X" &$ FnRootX DUP %+ P!
	"1-UTPT(" F@ DO>STR &$ ",X)-" &$ Q@ DO>STR &$ "/2" &$ FnRootX Y!

**	2FNROOT((1-UTPT(F,W))-P,P,1)\|>P:
**	FNROOT((1-UTPT(F,Y))-Q/2,Y,1)\|>Y:
    ;

    Method@ #2=
    IT
    ::	Type@

	ONE #=casedrop
	::
	    L4@ Y@ L8@ %* L14@ %SQRT %/ 2DUP
	    %- L!
	    %+ U!

**	    L0(4)-Y*L0(8)/\v/L0(14)\|>L:
**	    L0(4)+Y*L0(8)/\v/L0(14)\|>U:
	;

	TWO #=casedrop
	::
	    L9@ DUP %* L15@ %/ L10@ DUP %* L16@ %/ %+ %SQRT M!
	    L5@ L6@ %- N!
	    N@ Y@ M@ %* 2DUP
	    %- L!
	    %+ U!

**	    \v/(L0(9)\178/L0(15)+L0(10)\178/L0(16))\|>M:
**	    L0(5)-L0(6)\|>N:
**	    N-Y*M\|>L:
**	    N+Y*M\|>U:
	;

	THREE #=casedrop
	::
	    R@ %1 R@ %- %* L14@ %/ %SQRT M!
	    R@ Y@ M@ %* 2DUP
	    %- L!
	    %+ U!

**	    \v/(R*(1-R)/L0(14))\|>M:
**	    R-Y*M\|>L:
**	    R+Y*M\|>U:
	;

	FOUR #=casedrop
	::
	    L2@ L15@ %/ M!
	    L3@ L16@ %/ N!
	    M@ %1 M@ %- %* L15@ %/ N@ %1 N@ %- %* L16@ %/ %+ %SQRT O!
	    M@ N@ %- Y@ O@ %* 2DUP
	    %- L!
	    %+ U!

**	    L0(2)/L0(15)\|>M:
**	    L0(3)/L0(16)\|>N:
**	    \v/(M*(1-M)/L0(15)+N*(1-N)/L0(16))\|>O:
**	    M-N-Y*O\|>L:
**	    M-N+Y*O\|>U:
	;

	FIVE #=case
	::
	    L11@ L14@ %SQRT %/ M!
	    L4@ Y@ M@ %* 2DUP
	    %- L!
	    %+ U!

**	    L0(11)/\v/L0(14)\|>M:
**	    L0(4)-Y*M\|>L:
**	    L0(4)+Y*M\|>U:
	;
					( SIX )
	L5@ L6@ %- M!
	S@ %1 L15@ %/ %1 L16@ %/ %+ %SQRT %* N!
	M@ Y@ N@ %* 2DUP
	%- L!
	%+ U!

**	L0(5)-L0(6)\|>M:
**	S\v/(1/L0(15)+1/L0(16))\|>N:
**	M-Y*N\|>L:
**	M+Y*N\|>U:
    ;

    ( ** Display section ** )

    ( * Lines 1 & 2 * )

    " level: "
    Method@ #1= ITE
    ::  " Test" SWAP&$ Q@

	Type@

	THREE #=casedrop
	::
	    "     Test P: " L1@ L14@ %/

	    NAMELESS	_DoRnd
	    ::
		%-7 RNDXY
	    ;

**	    ROUND(L0(1)/L0(14),\1735)
	;

	FOUR #=case
	::
	    " Test P1-P2: " L2@ L15@ %/ L3@ L16@ %/ %- DoRnd

**	    ROUND(L0(2)/L0(15)-L0(3)/L0(16),\1735):
	;

	NAMELESS	_2NULL$
	::
	    NULL$ NULL$
	;
    ;
    ::	"Conf." SWAP&$ C@
	2NULL$
    ;

    ( * Lines 3 & 4 * )

    Method@ #1= ITE
    ::  "       Prob: " P@ %7 RNDXY	( ATTN: line 3 inserted below ! )
    ;
    ::
	"Range - min: " L@ DoRnd
	"        max: " U@ DoRnd
    ;

    ( * Lines 3 & 5 * )

    Method@ #1=

    Type@ FOUR #<=
    ITE
    ::  IT
	::  "     Test z: " Z@ DoRnd
	    2SWAP
	;

	" Critical z:"
    ;
    ::  IT
	::  "     Test t: " W@ DoRnd
	    2SWAP
	;

	" Critical t:"
    ;

    AltHyp@ #3=
    Method@ #2= OR
    ITE
    ::  CHR \B1 >T$ Y@ %ABS
    ;
    ::	APPEND_SPACE Y@
    ;

    DoRnd

    ( * Line 6 & 7 * )

    Method@ #1= ITE
    ::
	Type@

	ONE #=casedrop
	::
	    AltHyp@ #3= case
	    ::
		L8@
		NAMELESS	_Range
		::
		    2NULL$
		    "Range: "
		    L7@ Y@ 6ROLL %* L14@ %SQRT %/ 2DUP %- %-5 RNDXY DO>STR
		    " \8D " &$ UNROT
		    %+ %-5 RNDXY DO>STR &$
		;

**	        ROUND(L0(7)-Y*L0(8)/\v/L0(14),\1735)" \-> "
**		ROUND(L0(7)+Y*L0(8)/\v/L0(14),\1735)
	    ;

	    " Critical \81: "
	    L7@ Y@ L8@ %* L14@ %SQRT %/ %+ DoRnd

**	    ROUND(L0(7)+Y*L0(8)/\v/L0(14),\1736)

	    2NULL$
	;

	FIVE #=case
	::
	    AltHyp@ #3= case
	    ::
		L11@ Range

**	        ROUND(L0(7)-Y*L0(11)/\v/L0(14),\1735)" \-> "
**		ROUND(L0(7)+Y*L0(11)/\v/L0(14),\1735)
	    ;

	    " Critical \81: "
	    L7@ Y@ L11@ %* L14@ %SQRT %/ %+ DoRnd

**	    ROUND(L0(7)+Y*L0(11)/\v/L0(14),\1736)

	    2NULL$
	;

	2NULL$ 2NULL$
    ;
    ::	2NULL$ 2NULL$
    ;

    FOURTEEN {}N Screen!
;

*** _InputL *****************************************************************
* Interface :	( $t $l #tv --> )
* Description :
*	Questionaire support
*****************************************************************************

NAMELESS	_InputL
::
    DUP 4UNROLL TopicVarN@ 3PICKSWAP	( --> #tv $t $l $h % )

    BEGIN
	FOUR NDUP			( --> .. $t $l $h % )

	InputIFSpecs DoInputForm	( --> #tv .. f )

	NOTcase				( cancel )
	::  5DROP			( clean stack )
	    3RDROP			( cancel caller & leave )
	;

	DUPTYPEREAL?
	ITE
	    TRUE
	::
	    [#] argtypeerr GETTHEMESG DoAlert
	    DROPFALSE
	;
    UNTIL
					( --> #tv $t $l $h % %' )
    5UNROLL				( --> #tv %' $t $l $h % )
    4DROP SWAP				( --> %' #tv )
    TopicVarN!
;

*** _FnRootX ****************************************************************
* Interface :	( $exp --> % | cancel )
* Description :
*	Do FNROOT
*****************************************************************************

NAMELESS	_FnRootX
::
    exprparse NcaseTYPEERR
    "X" exprparse NcaseTYPEERR
    %1
    DOROOT
    DUPTYPEREAL? ?SEMI
    SETATTNERR
;

*** _ShowStack **************************************************************
* Interface :	( $title --> )
* Description :
*	Show stack with screen title
*****************************************************************************

**NAMELESS	_ShowStack
**::
**    DEPTH SWAPONE			( --> .. #n $ #1 )
**    3PICK #1+_ONE_DO (DO)
**	#1+				( --> .. $ #' )
**	ISTOP@ #1+PICK
**	ISTOP@ INDEX@#- #>$ CHR_: >T$	( --> .. $ #2 ob $1 )
**	SWAP DECOMP$ &$			( --> .. $ #2 $ )
**	SWAP				( --> .. $ $ #2 )
**    LOOP
**    {}N
**    '					( 5: Message Handler )
**    ::
****	LM_SETVIEWTYPE
**	SIXTY
**	#=casedrop TrueTrue		( full size window )
**	DROPFALSE
**    ;
**    OVER CARCOMP			( 4. Screen Title from 1st line )
**    ONE					( 3. simple string display )
**    4ROLL CDRCOMP			( 2. The Message )
**    ZERO				( 1: VIEW ONLY display box )
**    Choose
**    DROP				( drop VIEW ONLY exit flag, FALSE )
**    DROP				( --> .. )
**;

*****************************************************************************
*		End of Objects
*****************************************************************************

ASSEMBLE
codeEnd
RPL

*****************************************************************************
*		PARAMETER
*****************************************************************************

ASSEMBLE
	Vfield DirVar,DirCode
RPL
  {
	{
	    " Test level: " % .05
	    NULL$ NULL$
	    "    Test z : " % -2.353756
	    "      Prob : " % .0092924
	    " Critical z: " % -1.644854
	    " Critical \81: " % 48.60236
	}

	ONE			( Method -- ONE: Hypoth test,
					    TWO: Conf interval )
	ONE			( Type   -- ONE:   Z 1 u
					    TWO:   Z u1 - u2
					    THREE: Z 1 P
					    FOUR:  Z P1 - P2
					    FIVE:  T 1 u
					    SIX:   T u1 - u2 )
	ONE			( AltHyp -- ONE:   u < u0
					    TWO:   u > u0
					    THREE: u \=/ u0 )
	%8			( L1 )
	%8			( L2 )
	% 12			( L3 )
	% 48			( L4 )
	% 48			( L5 )
	% 52			( L6 )
	% 50			( L7 )
	% 3.8			( L8 )
	% 3.8			( L9 )
	% 3.5			( L10 )
	% 3.8			( L11 )
	% 3.8			( L12 )
	% 3.5			( L13 )
	% 20			( L14 )
	% 20			( L15 )
	% 45			( L16 )
	%.5			( L17 )
	% .05			( Q )
	% .99			( C )
  }


*****************************************************************************
*		AltViews
*****************************************************************************

ASSEMBLE
	Vfield DirCode,DirAlt
RPL
  { }


*****************************************************************************
*		ApLet note -- preset w/ (c)
*****************************************************************************

ASSEMBLE
	Vfield DirAlt,DirNot

	CON(5)	=DOCSTR
	REL(5)	noteEnd

	NIBASC	"Inference for the '38G\n"
	NIBASC	" '2000 Detlef Mller\n"
	NIBASC	"  <detlef@provi.de>\n"
	NIBASC	"  '2000 Colin Croft\n"
	NIBASC	" <ccroft@iinet.net.au>\n"
	NIBASC	"   $Revision: 1.8 $"
*		 1234567890123456789012
noteEnd
RPL


*****************************************************************************
*		SketchSet
*****************************************************************************

ASSEMBLE
	Vfield DirNot,DirEnd
RPL
  { }


******************************************************************************
*				End of INFER.S
******************************************************************************